@@ -21,7 +21,6 @@ SB = ShenBiharmonic
 SN = ShenNeumann
 CN = CombinedShenNeumann
 MN = MikNeumann
-TU = MixedTU
 DU = DirichletU
 UD = UpperDirichlet
 DN = DirichletNeumann
@@ -75,11 +74,11 @@ from __future__ import division
 import functools
 import numpy as np
 import sympy as sp
-from shenfun.optimization import cython, numba
-from shenfun.matrixbase import SpectralMatrix
+from shenfun.optimization import cython, numba, optimizer
+from shenfun.matrixbase import SpectralMatrix, SparseMatrix
 from shenfun.la import TDMA as generic_TDMA
 from shenfun.la import PDMA as generic_PDMA
-from .la import TDMA, PDMA, TwoDMA, FDMA
+from .la import TDMA, PDMA, TwoDMA, FDMA, ADD_Solve
 from . import bases
 
 x = sp.symbols('x', real=True)
@@ -93,7 +92,6 @@ SB = bases.ShenBiharmonic
 SN = bases.ShenNeumann
 CN = bases.CombinedShenNeumann
 MN = bases.MikNeumann
-TU = bases.MixedTU
 DU = bases.DirichletU
 UD = bases.UpperDirichlet
 DN = bases.DirichletNeumann
@@ -428,6 +426,14 @@ class BTTmat(SpectralMatrix):
         u[ss] = b[ss]*d[sl]
         return u
 
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        ck = get_ck(N, self.testfunction[0].quad)
+        sc = self.scale*Q.scale
+        return SparseMatrix({0: self.scale*np.pi/2*Q[0],
+                            -2: self.scale*ck[:-2]*np.pi/2*Q[-2],
+                             2: self.scale*np.pi/2*Q[2]}, (N, N), scale=sc)
+
 class BSNSNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -537,9 +543,23 @@ class BTSDmat(SpectralMatrix):
         Q = min(N, M)
         ck = get_ck(N, test[0].quad)
         d = {0: np.pi/2*ck[:Q]}
-        d[-2] = -np.pi/2*ck[2:(dmax(N, M, -2)+2)]
+        d[-2] = -np.pi/2
+        if test[0].quad == 'GL':
+            d[-2] = -np.pi/2*np.ones(Q)
+            d[-2][-1] *= 2
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        d = {0: Q[0]*self[0]}
+        d[0][:-2] += Q[2]*self[-2]
+        d[-2] = Q[0][2:]*self[-2]
+        d[2] = Q[2][:]*(self[0][2:])
+        d[2][:-2] += Q[4]*self[-2]
+        d[4] = Q[4][:]*self[0][4:]
+        sc = self.scale*Q.scale
+        return SparseMatrix(d, (N-2, N-2), scale=sc)
+
 
 class BTSNmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -567,6 +587,18 @@ class BTSNmat(SpectralMatrix):
               0: np.pi/2*ck[:-2]}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        k = np.ones(N)
+        k[:2] = 0
+        sc = self.scale*Q.scale
+        d = {0: Q[0]*self[0]}
+        d[0][:-2] += Q[2]*self[-2][:N-4]
+        d[-2] = Q[0][2:]*self[-2][:N-4]
+        d[2] = Q[2]*self[0][2:]
+        d[2][:-2] += Q[4]*self[-2][2:N-4]
+        d[4] = Q[4][:]*self[0][4:]
+        return SparseMatrix(d, (N-2, N-2), scale=sc)
 
 class BSBSBmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -1227,6 +1259,7 @@ class ASDSDmat(SpectralMatrix):
         d[2] = -4*np.pi*(k[:-4]+1)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self._matvec_methods += ['cython']
+        self.solver = ADD_Solve(self)
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -1244,53 +1277,6 @@ class ASDSDmat(SpectralMatrix):
 
         return c
 
-    def solve(self, b, u=None, axis=0):
-        N = self.shape[0] + 2
-        assert N == b.shape[0]
-        s = self.trialfunction[0].slice()
-
-        if u is None:
-            u = b
-        else:
-            assert u.shape == b.shape
-
-        # Move axis to first
-        if axis > 0:
-            u = np.moveaxis(u, axis, 0)
-            if u is not b:
-                b = np.moveaxis(b, axis, 0)
-
-        bs = b[s]
-        us = u[s]
-        if len(b.shape) == 1:
-            se = 0.0
-            so = 0.0
-        else:
-            se = np.zeros(us.shape[1:])
-            so = np.zeros(us.shape[1:])
-
-        d = self[0]
-        d1 = self[2]
-        M = us.shape
-        us[-1] = bs[-1] / d[-1]
-        us[-2] = bs[-2] / d[-2]
-        for k in range(M[0]-3, -1, -1):
-            if k%2 == 0:
-                se += us[k+2]
-                us[k] = bs[k] - d1[k]*se
-            else:
-                so += us[k+2]
-                us[k] = bs[k] - d1[k]*so
-            us[k] /= d[k]
-
-        u /= self.scale
-        self.testfunction[0].bc.set_boundary_dofs(u, True)
-        if axis > 0:
-            u = np.moveaxis(u, 0, axis)
-            if u is not b:
-                b = np.moveaxis(b, 0, axis)
-        return u
-
 
 class ASDSDmatW(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -1488,6 +1474,86 @@ class ATTmat(SpectralMatrix):
             c = super(ATTmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        k = np.ones(N)*np.pi/2
+        k[:2] = 0
+        sc = self.scale*Q.scale
+        return SparseMatrix({0: k}, (N, N), scale=sc)
+
+
+class ATSDmat(SpectralMatrix):
+    r"""Stiffness matrix for inner product
+
+    .. math::
+
+        A_{kj} = (\phi''_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N
+
+    and :math:`\psi_k` is the Chebyshev basis function and :math:`\phi_j`
+    is Shen's Dirichlet basis.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], T)
+        assert isinstance(trial[0], SD)
+        N = test[0].N
+        k = np.arange(N, dtype=float)
+        def _getkey(j):
+            return (k[j:-2]*(k[j:-2]**2-k[:-(j+2)]**2) -
+                    k[j+2:]*(k[j+2:]**2-k[:-(j+2)]**2))*np.pi/2.
+        d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
+        d[0] = -np.pi/2*k[2:]*(k[2:]**2-k[:-2]**2)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        sc = self.scale*Q.scale
+        return SparseMatrix({2: np.pi/2, 0: -np.pi/2}, (N-2, N-2), scale=sc)
+
+
+class ATSNmat(SpectralMatrix):
+    r"""Stiffness matrix for inner product
+
+    .. math::
+
+        A_{kj} = (\phi''_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N \text{ and } k = 0, 1, ..., N
+
+    and :math:`\psi_k` is the Chebyshev basis function and :math:`\phi_j`
+    is the ShenNeumann basis.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], T)
+        assert isinstance(trial[0], SN)
+        N = test[0].N
+        k = np.arange(N, dtype=float)
+        def _getkey(j):
+            return (k[j:-2]*(k[j:-2]**2-k[:-(j+2)]**2) - k[j:-2]**2/(k[j:-2]+2)*((k[j:-2]+2)**2-k[:-(j+2)]**2))*np.pi/2.
+        d = dict.fromkeys(np.arange(0, N-2, 2), _getkey)
+        d[0] = - k[:-2]**2/(k[:-2]+2)*((k[:-2]+2)**2-k[:-2]**2)*np.pi/2.
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+    def __quasi__(self, Q):
+        N = self.testfunction[0].N
+        k = np.arange(N)
+        sc = self.scale*Q.scale
+        return SparseMatrix({2: np.pi/2,
+                             0: -np.pi/2*(k[:-2]/(k[:-2]+2))**2},
+                            (N-2, N-2), scale=sc)
+
+
 class ACNCNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -1520,36 +1586,6 @@ class ACNCNmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self.solve = generic_TDMA(self)
 
-class ATUSDmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        A_{kj} = (\phi''_j, \psi_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, 2, ..., N-2 \text{ and } k = 0, 1, 2, ..., N-2
-
-    and :math:`\phi_k` is Shen's Dirichlet basis and :math:`\psi_j`
-    is the MixedTU basis. This is a Petrov-Galerkin matrix.
-    w = (1-x**2)
-
-    """
-    def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], TU)
-        assert isinstance(trial[0], SD)
-        N = test[0].N
-        k = np.arange(N, dtype=float)
-        if test[0].is_scaled():
-            d = {0: -0.5*np.pi*k[1:-1]*k[2:]}
-        else:
-            d = {0: -0.5*np.pi*k[1:-1]*k[2:]**2*(k[2:]+1)}
-        d[2] = -d[0][:-2].copy()
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
-        self.solver = TwoDMA(self)
 
 class ADUSDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -1628,49 +1664,18 @@ class ASDHHmat(SpectralMatrix):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], HH)
         N = test[0].N
-        k = np.arange(N, dtype=float)
+        k = np.arange(N)
         ck = get_ck(N, test[0].quad)
 
         if trial[0].is_scaled():
             d = {0: -np.pi/2*ck[:-2]*k[2:],
-                 2: np.pi/2*k[:-4]*k[1:-3]/(k[:-4]+3)}
+                 2: np.pi/2*k[:-4]*(k[1:-3]/(k[:-4]+3))}
         else:
             d = {0: -np.pi/2*ck[:-2]*k[2:]*k[1:-1],
                  2: np.pi/2*k[:-4]*k[1:-3]}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self.solver = TwoDMA(self)
 
-class ATUSNmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        A_{kj} = (\phi''_j, \psi_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, 2, ..., N-2 \text{ and } k = 0, 1, 2, ..., N-2
-
-    and :math:`\phi_k` is Shen's Neumann basis and :math:`\psi_j`
-    is the MixedTU basis. This is a Petrov-Galerkin matrix.
-    w = (1-x**2).
-
-    """
-    def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], TU)
-        assert isinstance(trial[0], SN)
-        N = test[0].N
-        k = np.arange(N, dtype=float)
-        if test[0].is_scaled():
-            d = {0: -(np.pi/2)*k[:-2]**2*(k[1:-1]/k[2:]),
-                 2: (np.pi/2)*k[1:-3]*k[2:-2]}
-        else:
-            d = {0: -(np.pi/2)*k[:-2]**2*k[1:-1]*(k[:-2]+3),
-                 2: (np.pi/2)*k[1:-3]*k[2:-2]**2*(k[2:-2]+1)}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
-        self.solver = TwoDMA(self)
 
 class ADUSNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -1735,42 +1740,6 @@ class AHHHHmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class BTUSDmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        B_{kj} = (\phi_j, \psi_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
-
-    and :math:`\phi_k` is Shen's Dirichlet basis and :math:`\psi_j`
-    is the MixedTU basis. This is a Petrov-Galerkin matrix.
-    w = (1-x**2).
-
-    """
-    def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], TU)
-        assert isinstance(trial[0], SD)
-        N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N, dtype=float)
-        d = {-2: -(np.pi/8),
-              0: (np.pi/8)*ck[:-2]+(np.pi/4)*(k[:-2]+2)/(k[:-2]+3),
-              2: -(np.pi/8)*(k[:-4]+1)/(k[:-4]+3)-(np.pi/4)*(k[:-4]+2)/(k[:-4]+3),
-              4: (np.pi/8)*(k[:-6]+1)/(k[:-6]+3)}
-        if not test[0].is_scaled():
-            d[-2] *= (k[2:-2]+2)*(k[2:-2]+3)
-            d[0] *= (k[:-2]+2)*(k[:-2]+3)
-            d[2] *= (k[:-4]+2)*(k[:-4]+3)
-            d[4] *= (k[:-6]+2)*(k[:-6]+3)
-
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
-        self.solver = FDMA(self)
 
 class BDUSDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -1985,46 +1954,6 @@ class ASNCNmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
         self.solver = TwoDMA(self)
 
-class BTUSNmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        B_{kj} = (\phi_j, \psi_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, 2, ..., N-2 \text{ and } k = 0, 1, 2, ..., N-2
-
-    and :math:`\phi_k` is Shen's Dirichlet basis and :math:`\psi_j`
-    is the MixedTU basis. This is a Petrov-Galerkin matrix.
-    w = (1-x**2).
-
-    """
-    def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], TU)
-        assert isinstance(trial[0], SN)
-        N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N, dtype=float)
-        d = {-2: -(np.pi/8)*(k[2:-2]-2)**2/k[2:-2]**2,
-              0: (np.pi/8)*(ck[:-2]+2*(k[:-2]/(k[:-2]+3))*(k[:-2]/(k[:-2]+2))),
-              2: -(np.pi/8)*(2*k[2:-2]/k[3:-1]+k[1:-3]/k[3:-1]*(k[2:-2]/k[4:])**2),
-              4: (np.pi/8)*k[1:-5]/k[3:-3]}
-        if not test[0].is_scaled():
-            d[-2] *= k[4:]*(k[4:]+1)
-            d[0] *= k[2:]*(k[2:]+1)
-            d[2] *= k[2:-2]*k[3:-1]
-            d[4] *= k[2:-4]*k[3:-3]
-        d[-2][0] = 0
-        d[0][0] = np.pi
-        d[2][0] = 0
-        d[4][0] = 0
-
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
-        self.solver = FDMA(self)
 
 class BDUSNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
@@ -2209,6 +2138,8 @@ mat = _ChebMatDict({
     ((SD, 0), (SD, 2), (-1, 1), (x**2-1)): functools.partial(ASDSDmatW, scale=-1, measure=(x**2-1)),
     ((SD, 0), (SD, 2), (-1, 1), (1-x**2)): functools.partial(ASDSDmatW, measure=(1-x**2)),
     ((T , 0), (T , 2)): ATTmat,
+    ((T , 0), (SD, 2)): ATSDmat,
+    ((T , 0), (SN, 2)): ATSNmat,
     ((SN, 0), (SN, 2)): ASNSNmat,
     ((CN, 0), (CN, 2)): ACNCNmat,
     ((SB, 0), (SB, 2)): ASBSBmat,
@@ -2222,10 +2153,6 @@ mat = _ChebMatDict({
     ((SD, 0), (HH, 2)): ASDHHmat,
     ((DU, 0), (SD, 2), (-1, 1), (1-x**2)): functools.partial(ADUSDmat, measure=(1-x**2)),
     ((DU, 0), (SD, 0), (-1, 1), (1-x**2)): functools.partial(BDUSDmat, measure=(1-x**2)),
-    ((TU, 0), (SD, 2), (-1, 1), (1-x**2)): functools.partial(ATUSDmat, measure=(1-x**2)),
-    ((TU, 0), (SD, 0), (-1, 1), (1-x**2)): functools.partial(BTUSDmat, measure=(1-x**2)),
-    ((TU, 0), (SN, 2), (-1, 1), (1-x**2)): functools.partial(ATUSNmat, measure=(1-x**2)),
-    ((TU, 0), (SN, 0), (-1, 1), (1-x**2)): functools.partial(BTUSNmat, measure=(1-x**2)),
     ((DU, 0), (SN, 2), (-1, 1), (1-x**2)): functools.partial(ADUSNmat, measure=(1-x**2)),
     ((DU, 0), (SN, 0), (-1, 1), (1-x**2)): functools.partial(BDUSNmat, measure=(1-x**2)),
     ((SB, 0), (SB, 4)): SSBSBmat,
