@@ -277,7 +277,7 @@ def inner(expr0, expr1, output_array=None, level=0):
         assert test.argument == 0
         space = test.function_space()
         if isinstance(trial, Array):
-            if trial.tensor_rank == 0:
+            if trial.tensor_rank == 0 and isinstance(test, BasisFunction):
                 output_array = space.scalar_product(trial, output_array)
                 return output_array
             trial = trial.forward()
@@ -310,8 +310,9 @@ def inner(expr0, expr1, output_array=None, level=0):
                 continue
             for test_j, b0 in enumerate(base_test):              # second index test
                 for trial_j, b1 in enumerate(base_trial):        # second index trial
-                    dV = sp.simplify(test_scale[vec_i][test_j]*trial_scale[vec_j][trial_j]*testspace.coors.get_sqrt_det_g()*g)
-                    dV = sp.refine(dV, testspace.coors._assumptions)
+                    dV = sp.simplify(test_scale[vec_i][test_j]*trial_scale[vec_j][trial_j]*testspace.coors.sg*g, measure=testspace.coors._measure)
+                    dV = testspace.coors.refine(dV)
+
                     assert len(b0) == len(b1)
                     trial_sp = trialspace
                     if isinstance(trialspace, (CompositeSpace, MixedFunctionSpace)): # could operate on a vector, e.g., div(u), where u is vector
@@ -332,7 +333,7 @@ def inner(expr0, expr1, output_array=None, level=0):
                             ts = trial_sp[i]
                             tt = test_sp[i]
                             msx = 'xyzrs'[i]
-                            msi = sp.simplify(dv[msx])
+                            msi = dv[msx]
 
                             # assemble inner product
                             AA = inner_product((tt, a), (ts, b), msi)
@@ -353,10 +354,13 @@ def inner(expr0, expr1, output_array=None, level=0):
                                 if BB:
                                     DM.append(BB)
                                     has_bcs = True
+                                else:
+                                    DM.append(0)
                             else:
                                 DM.append(AA)
 
                         sc = tt.broadcast_to_ndims(np.array([sc]))
+                        scb = tt.broadcast_to_ndims(np.array([scb]))
                         if len(M) == 1: # 1D case
                             M[0].global_index = (test_ind[test_j], trial_ind[trial_j])
                             M[0].scale = sc[0]
@@ -372,11 +376,14 @@ def inner(expr0, expr1, output_array=None, level=0):
                                 A.append(DM[0])
                             else:
                                 if len(trial_sp.get_nonhomogeneous_axes()) == 1:
-                                    A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    A.append(TPMatrix(DM, test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
                                 elif len(trial_sp.get_nonhomogeneous_axes()) == 2:
-                                    A.append(TPMatrix([M[0], DM[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
-                                    A.append(TPMatrix([DM[0], M[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
-                                    A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    if DM[1] != 0:
+                                        A.append(TPMatrix([M[0], DM[1]], test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    if DM[0] != 0:
+                                        A.append(TPMatrix([DM[0], M[1]], test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    if DM[0] != 0 and DM[1] != 0:
+                                        A.append(TPMatrix(DM, test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
 
     # At this point A contains all matrices of the form. The length of A is
     # the number of inner products. For each index into A there are ndim 1D
@@ -413,6 +420,7 @@ def inner(expr0, expr1, output_array=None, level=0):
         if isinstance(tpmat, TPMatrix):
             try:
                 tpmat.simplify_diagonal_matrices()
+
             except KeyError:
                 continue
 
