@@ -89,7 +89,7 @@ class FourierBase(SpectralBase):
                 rv = (sp.cos(theta), sp.sin(theta))
     """
 
-    def __init__(self, N, padding_factor=1, domain=(0, 2*np.pi), dtype=np.float,
+    def __init__(self, N, padding_factor=1, domain=(0, 2*np.pi), dtype=float,
                  dealias_direct=False, coordinates=None):
         self._k = None
         self._planned_axes = None  # Collapsing of axes means that this base can be used to plan transforms over several collapsed axes. Store the axes planned for here.
@@ -106,7 +106,7 @@ class FourierBase(SpectralBase):
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
             N = self.shape(False)
-        points = np.arange(N, dtype=np.float)*2*np.pi/N
+        points = np.arange(N, dtype=float)*2*np.pi/N
         if map_true_domain is True:
             points = self.map_true_domain(points)
         if weighted:
@@ -125,7 +125,7 @@ class FourierBase(SpectralBase):
             x = self.mesh(False, False)
         x = np.atleast_1d(x)
         if output_array is None:
-            output_array = np.zeros(x.shape, dtype=np.complex)
+            output_array = np.zeros(x.shape, dtype=complex)
 
         if self._k is None:
             self._k = self.wavenumbers(bcast=False)
@@ -147,7 +147,7 @@ class FourierBase(SpectralBase):
     def evaluate_basis_derivative_all(self, x=None, k=0):
         V = self.evaluate_basis_all(x=x)
         if k > 0:
-            l = self.wavenumbers(bcast=False, scaled=True)
+            l = self.wavenumbers(bcast=False, scaled=False, eliminate_highest_freq=False)
             V = V*((1j*l)**k)[np.newaxis, :]
         return V
 
@@ -173,7 +173,8 @@ class FourierBase(SpectralBase):
         return self.forward.output_array
 
     def apply_inverse_mass(self, array):
-        if not self.coors.is_cartesian: # mass matrix may not be diagonal
+        coors = self.tensorproductspace.coors if self.tensorproductspace else self.coors
+        if not coors.is_cartesian: # mass matrix may not be diagonal, or there is scaling
             return SpectralBase.apply_inverse_mass(self, array)
         return array
 
@@ -342,7 +343,7 @@ class R2C(FourierBase):
 
     def __init__(self, N, padding_factor=1., domain=(0., 2.*np.pi),
                  dealias_direct=False, coordinates=None):
-        FourierBase.__init__(self, N, padding_factor=padding_factor, dtype=np.float,
+        FourierBase.__init__(self, N, padding_factor=padding_factor, dtype=float,
                              domain=domain, dealias_direct=dealias_direct,
                              coordinates=coordinates)
         self.N = N
@@ -350,7 +351,7 @@ class R2C(FourierBase):
         self._xfftn_bck = fftw.irfftn
         self._sn = []
         self._sm = []
-        self.plan((int(padding_factor*N),), (0,), np.float, {})
+        self.plan((int(padding_factor*N),), (0,), float, {})
 
     def wavenumbers(self, bcast=True, scaled=False, eliminate_highest_freq=False):
         k = np.fft.rfftfreq(self.N, 1./self.N).astype(int)
@@ -386,6 +387,10 @@ class R2C(FourierBase):
         shape[self.axis] = shape[self.axis]//2 + 1
         return fftw.aligned(shape, dtype=dtype)
 
+    @staticmethod
+    def short_name():
+        return 'R2C'
+
     def slice(self):
         return slice(0, self.N//2+1)
 
@@ -394,7 +399,7 @@ class R2C(FourierBase):
             return self.N//2+1
         return int(np.floor(self.padding_factor*self.N))
 
-    def evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
+    def _evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
         if fast_transform is False:
             assert abs(self.padding_factor-1) < 1e-8
             P = self.evaluate_basis_all(x=x)
@@ -417,8 +422,10 @@ class R2C(FourierBase):
                     array += np.conj(np.dot(P[:, 1:], fc[tuple(s)])).real
 
                 output_array[:] = np.moveaxis(array, 0, self.axis)
-        else:
-            self.backward.xfftn(normalise_idft=False)
+            return
+        assert input_array is self.backward.tmp_array
+        assert output_array is self.backward.output_array
+        self.backward.xfftn(normalise_idft=False)
 
     def _truncation_forward(self, padded_array, trunc_array):
         if not id(trunc_array) == id(padded_array):
@@ -517,20 +524,26 @@ class C2C(FourierBase):
 
     def __init__(self, N, padding_factor=1, domain=(0., 2.*np.pi),
                  dealias_direct=False, coordinates=None):
-        FourierBase.__init__(self, N, padding_factor=padding_factor, dtype=np.complex,
+        FourierBase.__init__(self, N, padding_factor=padding_factor, dtype=complex,
                              domain=domain, dealias_direct=dealias_direct,
                              coordinates=coordinates)
         self.N = N
         self._xfftn_fwd = fftw.fftn
         self._xfftn_bck = fftw.ifftn
-        self.plan((int(padding_factor*N),), (0,), np.complex, {})
+        self.plan((int(padding_factor*N),), (0,), complex, {})
         self._slp = []
 
-    def evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
+    @staticmethod
+    def short_name():
+        return 'C2C'
+
+    def _evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
         if fast_transform is False:
-            SpectralBase.evaluate_expansion_all(self, input_array, output_array, x, False)
-        else:
-            self.backward.xfftn(normalise_idft=False)
+            SpectralBase._evaluate_expansion_all(self, input_array, output_array, x, False)
+            return
+        assert input_array is self.backward.tmp_array
+        assert output_array is self.backward.output_array
+        self.backward.xfftn(normalise_idft=False)
 
     def wavenumbers(self, bcast=True, scaled=False, eliminate_highest_freq=False):
         k = np.fft.fftfreq(self.N, 1./self.N).astype(int)
