@@ -40,7 +40,7 @@ class SpectralBase:
         self.axis = 0
         self.bc = None
         self.padding_factor = padding_factor
-        if padding_factor is not 1:
+        if padding_factor != 1:
             self.padding_factor = np.floor(N*padding_factor)/N if N > 0 else 1
         self.dealias_direct = dealias_direct
         self._mass = None         # Mass matrix (if needed)
@@ -128,6 +128,7 @@ class SpectralBase:
             uniform = False
         if uniform is False:
             X = self.points_and_weights(N=N, map_true_domain=map_true_domain)[0]
+
         else:
             d = self.domain
             X = np.linspace(d[0], d[1], N)
@@ -291,9 +292,9 @@ class SpectralBase:
                                self.backward.tmp_array)
 
         if kind == 'normal':
-            self.evaluate_expansion_all(self.backward.tmp_array,
-                                        self.backward.output_array,
-                                        fast_transform=fast_transform)
+            self._evaluate_expansion_all(self.backward.tmp_array,
+                                         self.backward.output_array,
+                                         fast_transform=fast_transform)
         else:
             if kind == 'uniform':
                 mesh = self.mesh(bcast=False, map_true_domain=False, uniform=True)
@@ -301,9 +302,9 @@ class SpectralBase:
                 mesh = kind.mesh()
                 if len(kind) > 1:
                     mesh = np.squeeze(mesh[self.axis])
-            self.evaluate_expansion_all(self.backward.tmp_array,
-                                        self.backward.output_array,
-                                        x=mesh, fast_transform=False) # cannot use fast transforms on random mesh
+            self._evaluate_expansion_all(self.backward.tmp_array,
+                                         self.backward.output_array,
+                                         x=mesh, fast_transform=False) # cannot use fast transforms on random mesh
 
         if output_array is not None:
             output_array[...] = self.backward.output_array
@@ -408,9 +409,9 @@ class SpectralBase:
         #warnings.warn('Using slow sympy evaluate_basis_derivative')
         if x is None:
             x = self.mesh(False, False)
+        x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape, dtype=self.dtype)
-        x = np.atleast_1d(x)
         X = sp.symbols('x', real=True)
         basis = self.sympy_basis(i=i, x=X).diff(X, k)
         output_array[:] = sp.lambdify(X, basis, 'numpy')(x)
@@ -441,7 +442,7 @@ class SpectralBase:
             V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
         return V
 
-    def evaluate_expansion_all(self, input_array, output_array,
+    def _evaluate_expansion_all(self, input_array, output_array,
                                x=None, fast_transform=False):
         r"""Evaluate expansion on 'x' or entire mesh
 
@@ -492,7 +493,7 @@ class SpectralBase:
         if output_array is None:
             output_array = np.zeros(x.shape, dtype=self.dtype)
         x = self.map_reference_domain(x)
-        self.evaluate_expansion_all(u, output_array, x, False)
+        self._evaluate_expansion_all(u, output_array, x, False)
         return output_array
 
     def _evaluate_scalar_product(self, fast_transform=False):
@@ -641,7 +642,7 @@ class SpectralBase:
             xfftn_fwd = plan_fwd(U, n, (axis,), threads=threads, flags=flags)
             V = xfftn_fwd.output_array
 
-            if np.issubdtype(dtype, np.floating):
+            if np.issubdtype(dtype, floating):
                 flags = (fftw.flag_dict[opts['planner_effort']],)
 
             xfftn_bck = plan_bck(V, n, (axis,), threads=threads, flags=flags, output_array=U)
@@ -651,7 +652,7 @@ class SpectralBase:
 
         self.axis = axis
 
-        if self.padding_factor is not 1:
+        if self.padding_factor != 1:
             trunc_array = self._get_truncarray(shape, V.dtype)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
             self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
@@ -740,7 +741,7 @@ class SpectralBase:
         """
         if forward_output:
             return self.N
-        if self.padding_factor is not 1:
+        if self.padding_factor != 1:
             return int(np.floor(self.padding_factor*self.N))
         return self.N
 
@@ -804,6 +805,15 @@ class SpectralBase:
     def has_nonhomogeneous_bcs(self):
         return False
 
+    @property
+    def use_fixed_gauge(self):
+        """Return True if space (or TensorProductSpace this space belongs to)
+        contains only Neumann boundary conditions"""
+        T = self.tensorproductspace
+        if T:
+            return T.use_fixed_gauge
+        return self.boundary_condition() == 'Neumann'
+
     @property
     def rank(self):
         """Return rank of function space
@@ -843,6 +853,10 @@ class SpectralBase:
     def family():
         return ''
 
+    @staticmethod
+    def short_name():
+        return ''
+
     def get_mass_matrix(self):
         mat = self._get_mat()
         dx = self.coors.get_sqrt_det_g()
@@ -852,7 +866,7 @@ class SpectralBase:
             msdict = split(dx)
             assert len(msdict) == 1
             dx = msdict[0]['xyzrs'[self.axis]]
-            if self.axis == 0:
+            if self.axis == self.tensorproductspace.dimensions-1:
                 dx *= msdict[0]['coeff']
         if not dx == 1:
             if not isinstance(dx, Number):
@@ -1333,9 +1347,11 @@ def inner_product(test, trial, measure=1):
         x0 = x0.pop()
         x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
         measure = measure.subs(x0, x)
-        if measure.subs(x, 1).evalf() < 0:
+        if measure.subs(x, (test[0].domain[0]+test[0].domain[1])/2).evalf() < 0:
             sc = -1
             measure *= sc
+        if measure.is_polynomial():
+            measure = sp.simplify(measure)
         key = key + (test[0].domain, measure)
 
     mat = test[0]._get_mat()
